@@ -1,7 +1,22 @@
-/* vmsfunctions.c */
+/* VMS functions
+Copyright (C) 1996-2013 Free Software Foundation, Inc.
+This file is part of GNU Make.
 
-#include "make.h"
+GNU Make is free software; you can redistribute it and/or modify it under the
+terms of the GNU General Public License as published by the Free Software
+Foundation; either version 3 of the License, or (at your option) any later
+version.
+
+GNU Make is distributed in the hope that it will be useful, but WITHOUT ANY
+WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR
+A PARTICULAR PURPOSE.  See the GNU General Public License for more details.
+
+You should have received a copy of the GNU General Public License along with
+this program.  If not, see <http://www.gnu.org/licenses/>.  */
+
+#include "makeint.h"
 #include "debug.h"
+#include "job.h"
 
 #ifdef __DECC
 #include <starlet.h>
@@ -16,15 +31,12 @@
 #ifdef HAVE_VMSDIR_H
 
 DIR *
-opendir (dspec)
-     char *dspec;
+opendir (char *dspec)
 {
-  struct DIR *dir  = (struct DIR *)xmalloc (sizeof (struct DIR));
-  struct NAM *dnam = (struct NAM *)xmalloc (sizeof (struct NAM));
+  struct DIR *dir  = xcalloc (sizeof (struct DIR));
+  struct NAM *dnam = xmalloc (sizeof (struct NAM));
   struct FAB *dfab = &dir->fab;
-  char *searchspec = (char *)xmalloc (MAXNAMLEN + 1);
-
-  memset (dir, 0, sizeof *dir);
+  char *searchspec = xmalloc (MAXNAMLEN + 1);
 
   *dfab = cc$rms_fab;
   *dnam = cc$rms_nam;
@@ -60,8 +72,7 @@ opendir (dspec)
   while (0)
 
 struct direct *
-readdir (dir)
-     DIR * dir;
+readdir (DIR *dir)
 {
   struct FAB *dfab = &dir->fab;
   struct NAM *dnam = (struct NAM *)(dfab->fab$l_nam);
@@ -77,7 +88,7 @@ readdir (dir)
 
   if (!((i = sys$search (dfab)) & 1))
     {
-      DB (DB_VERBOSE, (_("sys$search failed with %d\n"), i));
+      DB (DB_VERBOSE, (_("sys$search() failed with %d\n"), i));
       return (NULL);
     }
 
@@ -91,14 +102,16 @@ readdir (dir)
   dentry->d_namlen = dnam->nam$b_name + dnam->nam$b_type;
   strncpy (dentry->d_name, dnam->nam$l_name, dentry->d_namlen);
   dentry->d_name[dentry->d_namlen] = '\0';
+
+#ifdef HAVE_CASE_INSENSITIVE_FS
   uppercasify (dentry->d_name);
+#endif
 
   return (dentry);
 }
 
 int
-closedir (dir)
-     DIR *dir;
+closedir (DIR *dir)
 {
   if (dir != NULL)
     {
@@ -115,8 +128,7 @@ closedir (dir)
 #endif /* compiled for OpenVMS prior to V7.x */
 
 char *
-getwd (cwd)
-     char *cwd;
+getwd (char *cwd)
 {
   static char buf[512];
 
@@ -126,10 +138,12 @@ getwd (cwd)
     return (getcwd (buf, 512));
 }
 
+#if 0
+/*
+ * Is this used? I don't see any reference, so I suggest to remove it.
+ */
 int
-vms_stat (name, buf)
-     char *name;
-     struct stat *buf;
+vms_stat (char *name, struct stat *buf)
 {
   int status;
   int i;
@@ -224,10 +238,10 @@ vms_stat (name, buf)
 
   return 0;
 }
+#endif
 
 char *
-cvt_time (tval)
-     unsigned long tval;
+cvt_time (unsigned long tval)
 {
   static long int date[2];
   static char str[27];
@@ -245,17 +259,3 @@ cvt_time (tval)
 
   return (str);
 }
-
-int
-strcmpi (s1, s2)
-    const char *s1;
-    const char *s2;
-{
-  while (*s1 != '\0' && toupper(*s1) == toupper(*s2))
-    {
-      s1++;
-      s2++;
-    }
-
-  return toupper(*(unsigned char *) s1) - toupper(*(unsigned char *) s2);
-}
